/* --------------------------------------------------

   FAMICOM BASIC keyboard read driver.
   "kome wo kame"
   
   Programed by K.Kunikane. (rerofumi)
   
---------------------------------------------------- */

/* ----------------------- */
/* --- include             */
/* ----------------------- */
#include <m8c.h>
#include "fc2hid_table.h"

/* ----------------------- */
/* --- global work         */
/* ----------------------- */

/* --- data buffer */
BYTE keyBuffer[9*2];

/* --- endpoint buffer */
BYTE hidReport[8];

/* ----------------------- */
/* --- interface           */
/* ----------------------- */

/* --- GPIO initialize */
void InitializeKey(void)
{
	BYTE port;

	port = PRT0DR;
	port &= 0b01111111;
	port |= 0b01000000;
	PRT0DR = port;
}

/* --- GPIO read work */
void ReadKey(void)
{
	BYTE i, port;
	
	/* --- buffer swap */
	for(i=0; i<9; i++) {
		keyBuffer[i+9] = keyBuffer[i];
	}

	/* --- reset */
	port = PRT0DR;
	port |= 0b10000000;
	PRT0DR = port;
	//LCD_1_Delay50uTimes(2);
	port &= 0b01111111;
	PRT0DR = port;
	
	/* -- column */
	for(i=0; i<9; i++) {
		keyBuffer[i] = (PRT0DR << 4) & 0b11110000;
		port = PRT0DR;
		port &= 0b10111111;
		PRT0DR = port;
		port = PRT0DR & 0b00001111;
		keyBuffer[i] |= port;
		port = PRT0DR;
		port |= 0b01000000;
		PRT0DR = port;
	}
}

/* --- build HID Usage Report */
void SetHIDUsage(void)
{
	BYTE mod, index, report_ptr, usg, inkey;
	BYTE i, j;
	
	report_ptr = 2;
	index = 0;
	mod = 0;
	for(i=0; i<9; i++) {
		hidReport[i] = 0;
	}
	
	/* --- modifier key */
	/* left ctrl */
	if ((keyBuffer[7] & 0x08) != 0) {
	  mod |= 0x01;
	}
	/* left shift */
	if ((keyBuffer[7] & 0x10) != 0) {
	  mod |= 0x02;
	}
	/* left alt (grph) */
	if ((keyBuffer[7] & 0x20) != 0) {
	  mod |= 0x04;
	}
	/* right shift */
	if ((keyBuffer[0] & 0x20) != 0) {
	  mod |= 0x20;
	}
	hidReport[0] = mod;
	
	/* --- other key table */
	for(j=0; j<9; j++) {
		inkey = keyBuffer[j];
		for(i=0; i<8; i++) {
			if (report_ptr < 8) {
				if ((inkey & 0x01) != 0) {
					usg = key_usage[index];
					if (usg != 0xff) {
						hidReport[report_ptr] = usg;
						report_ptr++;
					}
				}
			}
			index++;
			inkey >>= 1;
		}
	}

}

